program plc_parcon;
{parcon system controlling a mitsubishi FX-PLC}
uses crt,dos;

const
     enq=#$05;
     ack=#$06;
     nak=#$15;
     stx=#$02;
     etx=#$03;
     com_base_adds : array[1..4] of word =($3f8,$2f8,$3e8,$2e8);

type
    block_type = array[0..63] of byte;

var
   Data_regs : array [0..99] of word; { D200-D299 in PLC }
   tempstr : string;
   ch : char;
   i,j : word;
   regs : registers;
   buffer : array[0..1000] of char;
   buffer_cnt : integer;
   data_block : block_type;
   base : word;
   com_port : word;
   address : word;

{$i hex}
(**************************************************************************)
(**************************************************************************)
(**************************************************************************)

procedure dtr_on;
begin
     port[base+4]:=0;
end;
procedure dtr_off;
begin
     port[base+4]:=1;
end;

function status : byte;
begin
     status:=port[base+6];
end;

function skeypressed : boolean;
begin
     skeypressed:=odd(port[base+5]);
end;
(**************************************************************************)

function sreadkey : char;
var
   i : word;
   ch : char;
begin
     i:=0;
     repeat
           inc(i);
     until skeypressed or (i>10000);
     if i<10000 then
        sreadkey:=chr(port[base])
     else begin
          sreadkey:=nak;
          Write(chr(7));
          end;
end;

procedure swrite(ch : char);
var
   v : word;
begin
     repeat
     until status and $2=0;
     repeat until port[base+5] and $20=$20;
     port[base]:=ord(ch);

end;
(**************************************************************************)
(**************************************************************************)



procedure frame_IO(str : string);
var
   i,j : integer;
begin
     for i:=0 to 200 do
         buffer[i]:=chr(0);
     dtr_on;
     for i:=1 to length(str) do
     begin
          Swrite(str[i]);
     end;
     dtr_off;
     i:=0;
     repeat
     begin
          inc(i);
          buffer[i]:=sreadkey;
     end;
     until (buffer[i]=etx) or (buffer[i]=ack) or (buffer[i]=nak);
     if buffer[i]=etx then
     begin
          inc(i);
          buffer[i]:=sreadkey;
          inc(i);
          buffer[i]:=sreadkey;
     end;
     buffer_cnt:=i;
end;
(**************************************************************************)
(**************************************************************************)


function read_frame : string;
var
   i : integer;
   str : string;
begin
     str:='';
     for i:=1 to buffer_cnt do
          str:=str+buffer[i];
     read_frame:=str;

end;

function un_frame(var block : string) : string;
{ takes a frame}
{ returns the data string or '' if an error occured }
var
   i: integer;
   sum : word;
   str : string;

begin
     sum:=0;
     for i:=2 to length(block)-2 do
         sum:=sum+ord(block[i]);
     str:=copy(block,length(block)-1,2);
     if hex2(sum mod 256)=str then
        str:=copy(block,2,length(block)-4)
        else
            str:='';
     un_frame:=str;
end;

function frame(cmd_data : string) : string;
{ takes a command and data string }
{ returns a complete frame with checksum & header }
var
   i : integer;
   sum : word;
   str : string;
begin
     sum:=ord(etx);
     for i:=1 to length(cmd_data) do
         sum:=sum+ord(cmd_data[i]);
     frame:=stx+cmd_data+etx+hex2(sum mod 256);
end;

function link_ok : boolean;
{ takes nothing}
{ returns true if PLC ACKnowledges ENQ from pc }
var
   ch : char;

begin
     dtr_on;
     swrite(enq);
     dtr_off;
     ch:=sreadkey;
     link_ok:=(ch=ack);
end;

(**************************************************************************)
(**************************************************************************)

function read_block(start : word; length : word) : string;
var
   s1 : string;
begin
     frame_IO(frame('0'+hex4(start)+hex2(length)));
     s1:=read_frame;
     read_block:=un_frame(s1);
end;

function Write_block(start : word; length : word; data : block_type) : string;
var
   s1 : string;
   i : integer;
begin
     s1:='1'+hex4(start)+hex2(length);
     for i:=1 to length do
         s1:=s1+hex2(data[i-1]);
     frame_IO(frame(s1));
     if buffer[1]=ack then
        Write_block:=' Block Accepted.'
     else
         Write_block:='';
end;
(**************************************************************************)

function force_on(address : word) : string;
begin
     frame_IO(frame('7'+hex4(swap(address))));
     if buffer[1]=ack then force_on:='ON'
     else force_on:='';
end;

function force_off(address : word) : string;
begin
     frame_IO(frame('8'+hex4(swap(address))));
     if buffer[1]=ack then force_off:='OFF'
     else force_off:='';
end;
(**************************************************************************)
(**************************************************************************)
procedure read_data_regs;
{ read the current settings of the data registers D200-D299
  from the PLC into data_regs[] }
{ max block size = 64 bytes, so must read 200 bytes in 4 blocks (of 50!) }
var
   i,j,k : integer;
   address : word;
   tstr,s1 : string;
   v : word;

begin
     address:=$1190; {D200}
     for i:=0 to 3 do
     begin
          tstr:=read_block(address+50*i,50);
          if tstr='' then
          begin
               dec(i);
{
               Writeln(' Reading Block Rejected ');
}
          end
          else begin
                    for j:=0 to 24 do
                    begin
                         s1:='$'+copy(tstr,4*j+1,4);
                         val(s1,v,k);
                         if k=0 then data_regs[i*25+j]:=v;
                    end;
               end;
     end;
end;

procedure Write_data_regs;
{ Write the current settings of data_regs[] to the data registers D200-D299
  in the PLC }
{ max block size = 64 bytes, so must write 200 bytes in 4 blocks (of 50!) }
var
   i,j,k : integer;
   address : word;
   tstr,s1 : string;
   v : word;

begin
     address:=$1190; {D200}
     for i:=0 to 3 do
     begin
          for j:=0 to 24 do
          begin
               data_block[j*2]:=hi(data_regs[i*25+j]);
               data_block[j*2+1]:=lo(data_regs[i*25+j]);
          end;
          tstr:=Write_block(address+50*i,50,data_block);
          if tstr='' then
          begin
               dec(i);
{
               Writeln(' Write Block Rejected.');
}
          end
          else begin
               end;
     end;
end;
(**************************************************************************)
(**************************************************************************)
procedure initialise;
var
   ch : char;
   regs : registers;

begin
     com_port:=2;
     base:=com_base_adds[com_port];
     regs.ah:=0;
     regs.dx:=com_port-1;
     regs.al:=$FA;
     intr($14,regs);
     if skeypressed then ch:=sreadkey;
end;
(**************************************************************************)
(**************************************************************************)
(**************************************************************************)

begin
     initialise;
     if link_ok then
     begin
          if paramcount>0 then
          tempstr:=paramstr(1);
          case tempstr[1] of
               '0' : begin
                          Writeln(' Reading data regs');
                          read_data_regs;
                     end;
               '1' : begin
                          Writeln(' Writing data regs');
                          write_data_regs;
                     end;
               '2' : Writeln(force_on($500));
               '3' : writeln(force_off($500));
          end;
{
          for i:=0 to 4 do
          begin
                for j:=0 to 99 do
                    data_regs[j]:=i*100+j;
                Write_data_regs;
                Writeln(' First data =',hex4(i*100));
                Writeln(' Last data  =',hex4(i*100+99));
                read_data_regs;
                for j:=0 to 99 do
                    if data_regs[j]<>i*100+j then
                    begin

                         Writeln(' Corrupted data at ',j);

                    end;
           end;

           address:=$500;
           for i:=0 to 23 do
           begin
                Writeln('Forcing ',hex4(address+i),' ',Force_on(address+i));
                delay(100);
                Writeln('Forcing ',hex4(address+i),' ',Force_off(address+i));
                delay(100);
           end;
}
     end
     else Writeln(' Link Failed.');

end.